#! /usr/bin/env node

require("./seleniumLoader.js");
var parseArgs = require("minimist");
var fs = require('fs');
var argv = process.argv.slice(2);

// unshortened options: 'stabilization-delay', 'base-url', 'sauce-user', 'sauce-key', 'token'
var aliases = {
	't' : 'testId',
	'l' : 'label',
	'd' : 'driver',
	'u' : 'url',
	'r' : 'report-file',
	'b' : 'browser',
    'p' : 'project'
};

var options = parseArgs(argv, { 'alias' : aliases});
var logger = TST.Logger.getLogger("selenium test runner");

function runTests(testIds){
	"use strict";

	function createResultsReport(testResults){
		return TST.TestReporter.report(testResults, TST.SettingsManager.getSetting("background.server.project", "Testim"));
	}

	function reportResults(reportText){
		return new Promise(function(resolve){
			if(options['report-file']){
				fs.writeFile(options['report-file'], reportText, function(err){
					if(err) {
						logger.error("could not save report file", options['report-file'], err);
					} else {
						logger.log("file save to", options['report-file']);
					}
					resolve();
				});
			} else {
				logger.log(reportText);
				resolve();
			}
		});
	}

	function exitProgram(){
		setTimeout(function(){
			// we should return 1 on fatal error
			global.process.exit(0);
		}, 3000);
	}

	// code starts here
	logger.log("got tests to run", testIds);
	TST.Selenium.testRunner.executeTests(testIds)
		.then(createResultsReport)
		.then(reportResults)
		.then(exitProgram);
}

function getTestToRunByTestId(testIdArg){
	"use strict";

	if(!testIdArg) {
		return [];
	}
	if(typeof testIdArg === "string"){
		return [testIdArg];
	}
	return testIdArg;
}

if(options['base-url']) {
	TST.Selenium.uesr_override_config = {
		baseUrl: options['base-url']
	};
}


TST.Selenium.stabilizationDelay = options['stabilization-delay'];
TST.Selenium.driverInfo = {
	driverType : options.driver || 'webdriver',
	url : options.url,
	browser: options.browser,
	sauceUser : options['sauce-user'],
	sauceKey : options['sauce-key']
};

if(!options.label && options.testId) {
	console.error("no test id or label, only got", options);
	setTimeout(function(){
		global.process.exit(1);
	}, 3000);
} else {
	TST.authenticationManager.loginWithApiKey(options["token"]).then(function(){
		if(options.project) {
			TST.ProfileManager.selectProfile(options.project);
		}
		if(options.label) {
			var labelsToRun;
			if(typeof options.label=== "string"){
				labelsToRun = [options.label];
			} else {
				labelsToRun = options.label;
			}
			TST.ScenarioServer.getTestsByLabels(labelsToRun).then(function(testsToRun){
				"use strict";
				var testIdToRun = testsToRun.concat(getTestToRunByTestId(options.testId));
				runTests(testIdToRun);
			});
		} else if(options.testId){
			runTests(getTestToRunByTestId(options.testId));
		} else {
			logger.error("no test to run");
			global.process.exit(0);
		}
	}).catch(function(err){
		logger.error("Failedd to login", options["token"], err);
	});
}